#[cfg(feature = "server")]
use super::surrealdb::DB;
use dioxus::prelude::*;
use serde::{Deserialize, Serialize};
#[cfg(feature = "server")]
use surrealdb::sql::Thing;

#[derive(Debug, Serialize, Deserialize)]
pub struct User {
    email: String,
    password: String,
}

#[cfg(feature = "server")]
#[derive(Debug, Deserialize)]
struct Record {
    id: Thing,
}

#[server(Register)]
pub async fn register(email: String, password: String) -> Result<String, ServerFnError> {
    tracing::info!("Creating new user");

    let mut res = DB
        .query("CREATE user SET email = $email, password = crypto::argon2::generate($password)")
        .bind(("email", email))
        .bind(("password", password))
        .await?;

    let user: Option<Record> = res.take(0)?;

    match user {
        Some(Record { id }) => {
            tracing::info!("Created new user ({id})");
            Ok(id.to_string())
        }
        _ => Err(ServerFnError::ServerError("Could not get id".to_string())),
    }
}

#[server(Signin)]
pub async fn signin(email: String, password: String) -> Result<User, ServerFnError> {
    let mut res = DB
        .query("SELECT * FROM user WHERE email = $email AND crypto::argon2::compare(password, $password)")
        .bind(("email", email))
        .bind(("password", password))
        .await?;

    let user: Option<User> = res.take(0)?;

    match user {
        Some(u) => {
            tracing::info!("User ({}) has signed in", u.email);
            Ok(u)
        }
        _ => Err(ServerFnError::ServerError("Could not get id".to_string())),
    }
}
